import logging
import json
import mysql.connector
from mysql.connector import errorcode
from src.constants import DBNAME, DBHOSTNAME, DBUSERNAME, DBPASSWORD

# Configure logging for debugging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

def get_connection():
    """Creates and returns a database connection."""
    try:
        conn = mysql.connector.connect(
            host=DBHOSTNAME,
            user=DBUSERNAME,
            password=DBPASSWORD,
            database=DBNAME
        )
        logging.info("Connected to the database successfully.")
        return conn
    except mysql.connector.Error as err:
        if err.errno == errorcode.ER_ACCESS_DENIED_ERROR:
            logging.error(f"Invalid database credentials. -{DBHOSTNAME}, {DBUSERNAME}, {DBPASSWORD}, {DBNAME}-  {err}")
        elif err.errno == errorcode.ER_BAD_DB_ERROR:
            logging.error("Database does not exist.")
        else:
            logging.error(f"Database connection error: {err}")
        return None


# Function to SELECT data from any table
def select_from_table(table, *args):
    """Fetches data from a given table.
    Returns:
        list(dict())
    """
    conn = get_connection()

    try:
        cursor = conn.cursor(dictionary=True)
        query = f"SELECT {', '.join(args) if args else '*'} FROM {table}"
        cursor.execute(query)
        results = cursor.fetchall()
        return results
    except mysql.connector.Error as err:
        logging.error(f"Error fetching data: {err}")
        return []
    finally:
        cursor.close()
        conn.close()

# Function to SELECT data with a WHERE condition
def select_from_table_where(table:str, condition:str, selection:str):
    """Fetches data from a table with a WHERE condition.
    
    Arguments:
        table (str): database table name, eg; "tblUsers"
        condition (str): condition string in SQL format, eg; "WHERE username = %s AND password_hash = %s"
        *columns: excepts all aditinal arguments and converts them into SQL query following SELECT as arguments, if none SQL statemnt uses *.
    Returns:
        list(dict())
    """
    conn = get_connection()
    if not conn:
        return []

    try:
        #logging.debug(table, condition, selection)
        cursor = conn.cursor(dictionary=True)
        query = f"SELECT {selection} FROM {table} WHERE {condition}"
        logging.debug(f"query='{query}'")
        cursor.execute(query)
        results = cursor.fetchall() # returns string in format str(dict(key:values))
        if not results:
            logging.debug("SQL return empty.")
            return None
        logging.debug(f"SQL results={results}")
        return results
    except mysql.connector.Error as err:
        logging.error(f"Error fetching data with condition: {err}")
        return None
    #finally:
    #    cursor.close()
    #    conn.close()

# Function to INSERT data into any table
def insert_into_table(table:str, data:dict):
    """Inserts data into a specified table."""
    conn = get_connection()
    if not conn:
        return False

    try:
        cursor = conn.cursor()
        columns = ', '.join(data.keys())
        values = ', '.join(['%s'] * len(data))
        query = f"INSERT INTO {table} ({columns}) VALUES ({values})"
        logging.debug(f"query='{query}', data.values={tuple(data.values())}")
        cursor.execute(query, tuple(data.values()))
        conn.commit()
        logging.info("Data inserted successfully.")
        return True
    except mysql.connector.Error as err:
        logging.error(f"Error inserting data: {err}")
        return False
    finally:
        cursor.close()
        conn.close()

# Function to DELETE data
def delete_from_table(table, condition):
    """Deletes data from a specified table based on a condition."""
    conn = get_connection()
    if not conn:
        return False

    try:
        cursor = conn.cursor()
        query = f"DELETE FROM {table} WHERE {condition}"
        cursor.execute(query)
        conn.commit()
        logging.info("Data deleted successfully.")
        return True
    except mysql.connector.Error as err:
        logging.error(f"Error deleting data: {err}")
        return False
    finally:
        cursor.close()
        conn.close()

# Function to UPDATE data
def update_table(table, data, condition):
    """Updates data in a specified table based on a condition."""
    conn = get_connection()
    if not conn:
        return False

    try:
        cursor = conn.cursor()
        set_clause = ', '.join([f"{key} = %s" for key in data.keys()])
        query = f"UPDATE {table} SET {set_clause} WHERE {condition}"
        data_arry = tuple(data.values())
        logging.debug(f"query='{query}', data.values()={data_arry}")
        cursor.execute(query, data_arry)
        conn.commit()
        logging.info("Data updated successfully.")
        return True
    except mysql.connector.Error as err:
        logging.error(f"Error updating data: {err}")
        return False
    finally:
        cursor.close()
        conn.close()


